@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2011-2014 Codership Oy <info@codership.com>
+// Copyright (C) 2011-2017 Codership Oy <info@codership.com>
 //
 
 #ifndef GALERA_IST_PROTO_HPP
@@ -12,6 +12,7 @@
 #include "gu_logger.hpp"
 #include "gu_serialize.hpp"
 #include "gu_vector.hpp"
+#include "gu_array.hpp"
 
 //
 // Message class must have non-virtual destructor until
@@ -420,7 +421,7 @@ namespace galera
                 const bool rolled_back(buffer.seqno_d() == -1);
 
                 galera::WriteSetIn ws;
-                boost::array<asio::const_buffer, 3> cbs;
+                gu::array<asio::const_buffer, 3>::type cbs;
                 size_t      payload_size; /* size of the 2nd cbs buffer */
                 size_t      sent;
 
@@ -549,8 +550,18 @@ namespace galera
                         trx->unserialize(&wbuf[0], wbuf.size(), 0);
                     }
 
-                    trx->set_received(0, -1, seqno_g);
-                    trx->set_depends_seqno(seqno_d);
+                    if (seqno_d == WSREP_SEQNO_UNDEFINED ||
+                        trx->version() < 3)
+                    {
+                        trx->set_received(0, -1, seqno_g);
+                        trx->set_depends_seqno(seqno_d);
+                    }
+                    else
+                    {
+                        trx->set_received_from_ws();
+                        assert(trx->global_seqno() == seqno_g);
+                        assert(trx->depends_seqno() >= seqno_d);
+                    }
                     trx->mark_certified();
 
                     log_debug << "received trx body: " << *trx;
