@@ -124,7 +124,7 @@ galera::ReplicatorSMM::ParseOptions::ParseOptions(Replicator&       repl,
                                                   gu::Config&       conf,
                                                   const char* const opts)
 {
-    conf.parse(opts);
+    if (opts) conf.parse(opts);
 
     if (conf.get<bool>(Replicator::Param::debug_log))
     {
