@@ -1,15 +1,15 @@
 /*
- * Copyright (C) 2010-2014 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2020 Codership Oy <info@codership.com>
  */
-#define __STDC_FORMAT_MACROS
 
 #include "../src/galera_service_thd.hpp"
 #include "../src/replicator_smm.hpp"
+
+#include "gu_inttypes.hpp"
+
 #include <check.h>
 #include <errno.h>
 
-#include <inttypes.h>
-
 namespace
 {
     class TestEnv
@@ -20,7 +20,7 @@ namespace
             GCache_setup(gu::Config& conf) : name_("service_thd_check.gcache")
             {
                 conf.set("gcache.name", name_);
-                conf.set("gcache.size", "4M");
+                conf.set("gcache.size", "1M");
                 log_info << "conf for gcache: " << conf;
             }
 
@@ -61,7 +61,7 @@ START_TEST(service_thd1)
 {
     TestEnv env;
     ServiceThd* thd = new ServiceThd(env.gcs(), env.gcache());
-    fail_if (thd == 0);
+    ck_assert(thd != 0);
     delete thd;
 }
 END_TEST
@@ -75,7 +75,7 @@ START_TEST(service_thd2)
     TestEnv env;
     DummyGcs& conn(env.gcs());
     ServiceThd* thd = new ServiceThd(conn, env.gcache());
-    fail_if (thd == 0);
+    ck_assert(thd != 0);
 
     conn.set_last_applied(0);
 
@@ -83,24 +83,24 @@ START_TEST(service_thd2)
     thd->report_last_committed (seqno);
     thd->flush();
     WAIT_FOR(conn.last_applied() == seqno);
-    fail_if (conn.last_applied() != seqno,
-             "seqno = %" PRId64 ", expected %" PRId64, conn.last_applied(),
-             seqno);
+    ck_assert_msg(conn.last_applied() == seqno,
+                  "seqno = %" PRId64 ", expected %" PRId64,
+                  conn.last_applied(), seqno);
 
     seqno = 5;
     thd->report_last_committed (seqno);
     thd->flush();
     WAIT_FOR(conn.last_applied() == seqno);
-    fail_if (conn.last_applied() != seqno,
-             "seqno = %" PRId64 ", expected %" PRId64, conn.last_applied(),
-             seqno);
+    ck_assert_msg(conn.last_applied() == seqno,
+                  "seqno = %" PRId64 ", expected %" PRId64,
+                  conn.last_applied(), seqno);
 
     thd->report_last_committed (3);
     thd->flush();
     WAIT_FOR(conn.last_applied() == seqno);
-    fail_if (conn.last_applied() != seqno,
-             "seqno = %" PRId64 ", expected %" PRId64, conn.last_applied(),
-             seqno);
+    ck_assert_msg(conn.last_applied() == seqno,
+                  "seqno = %" PRId64 ", expected %" PRId64,
+                  conn.last_applied(), seqno);
 
     thd->reset();
 
@@ -108,9 +108,9 @@ START_TEST(service_thd2)
     thd->report_last_committed (seqno);
     thd->flush();
     WAIT_FOR(conn.last_applied() == seqno);
-    fail_if (conn.last_applied() != seqno,
-             "seqno = %" PRId64 ", expected %" PRId64, conn.last_applied(),
-             seqno);
+    ck_assert_msg(conn.last_applied() == seqno,
+                  "seqno = %" PRId64 ", expected %" PRId64,
+                  conn.last_applied(), seqno);
 
     delete thd;
 }
@@ -120,7 +120,7 @@ START_TEST(service_thd3)
 {
     TestEnv env;
     ServiceThd* thd = new ServiceThd(env.gcs(), env.gcache());
-    fail_if (thd == 0);
+    ck_assert(thd != 0);
     // so far for empty GCache the following should be a noop.
     thd->release_seqno(-1);
     thd->release_seqno(2345);
@@ -138,6 +138,7 @@ Suite* service_thd_suite()
     tcase_add_test  (tc, service_thd1);
     tcase_add_test  (tc, service_thd2);
     tcase_add_test  (tc, service_thd3);
+    tcase_set_timeout(tc, 60);
     suite_add_tcase (s, tc);
 
     return s;
