@@ -8,6 +8,7 @@
 #include "gu_alloc.hpp"
 #include "gu_throw.hpp"
 #include "gu_assert.hpp"
+#include "gu_arch.h"
 #include "gu_limits.h"
 
 #include <sstream>
@@ -17,6 +18,7 @@
 gu::Allocator::HeapPage::HeapPage (page_size_type const size) :
     Page (static_cast<byte_t*>(::malloc(size)), size)
 {
+    assert(0 == (uintptr_t(base_ptr_) % GU_WORD_BYTES));
     if (0 == base_ptr_) gu_throw_error (ENOMEM);
 }
 
@@ -52,7 +54,8 @@ gu::Allocator::FilePage::FilePage (const std::string& name,
     fd_  (name, size, false, false),
     mmap_(fd_, true)
 {
-    base_ptr_ = reinterpret_cast<byte_t*>(mmap_.ptr);
+    base_ptr_ = static_cast<byte_t*>(mmap_.ptr);
+    assert(0 == (uintptr_t(base_ptr_) % GU_WORD_BYTES));
     ptr_      = base_ptr_;
     left_     = mmap_.size;
 }
@@ -167,7 +170,7 @@ gu::Allocator::alloc (page_size_type const size, bool& new_page)
 gu::Allocator::BaseNameDefault const gu::Allocator::BASE_NAME_DEFAULT;
 
 gu::Allocator::Allocator (const BaseName&         base_name,
-                          byte_t*                 reserved,
+                          void*                   reserved,
                           page_size_type          reserved_size,
                           heap_size_type          max_ram,
                           page_size_type          disk_page_size)
@@ -184,6 +187,7 @@ gu::Allocator::Allocator (const BaseName&         base_name,
     size_         (0)
 {
     assert (NULL != reserved || 0 == reserved_size);
+    assert (0 == (uintptr_t(reserved) % GU_WORD_BYTES));
     assert (current_page_ != 0);
     pages_->push_back (current_page_);
 }
