@@ -26,6 +26,8 @@
 #include "gu_histogram_test.hpp"
 #include "gu_stats_test.hpp"
 #include "gu_thread_test.hpp"
+#include "gu_asio_test.hpp"
+#include "gu_deqmap_test.hpp"
 
 typedef Suite *(*suite_creator_t)(void);
 
@@ -47,6 +49,8 @@ static suite_creator_t suites[] =
     gu_histogram_suite,
     gu_stats_suite,
     gu_thread_suite,
+    gu_asio_suite,
+    gu_deqmap_suite,
     0
 };
 
