@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2018 Codership Oy <info@codership.com>
+ * Copyright (C) 2009-2021 Codership Oy <info@codership.com>
  */
 
 #include "GCache.hpp"
@@ -24,30 +24,40 @@ namespace gcache
     }
 
     bool
-    GCache::discard_seqno (int64_t seqno)
+    GCache::discard_seqno (seqno_t seqno)
     {
 #ifndef NDEBUG
-        seqno_t begin(0);
+        seqno_t const begin(params.debug() ?
+                            (seqno2ptr.empty() ?
+                             SEQNO_NONE : seqno2ptr.index_begin()) : SEQNO_NONE);
         if (params.debug())
         {
-            begin = (seqno2ptr.begin() != seqno2ptr.end() ?
-                     seqno2ptr.begin()->first : 0);
             assert(begin > 0);
             log_info << "GCache::discard_seqno(" << begin << " - "
                      << seqno << ")";
         }
 #endif
-        for (seqno2ptr_t::iterator i = seqno2ptr.begin();
-             i != seqno2ptr.end() && i->first <= seqno;)
+        /* if we can't complete the operation, let's not even start */
+        if (seqno >= seqno_locked)
         {
-            BufferHeader* bh(ptr2BH (i->second));
+#ifndef NDEBUG
+            if (params.debug())
+            {
+                log_info << "GCache::discard_seqno(" << begin << " - " << seqno
+                         << "): " << seqno_locked << " is locked, bailing out.";
+            }
+#endif
+            return false;
+        }
+
+        while (seqno2ptr.index_begin() <= seqno && !seqno2ptr.empty())
+        {
+            BufferHeader* const bh(ptr2BH(seqno2ptr.front()));
 
             if (gu_likely(BH_is_released(bh)))
             {
-                assert (bh->seqno_g == i->first);
+                assert (bh->seqno_g == seqno2ptr.index_begin());
                 assert (bh->seqno_g <= seqno);
-
-                seqno2ptr.erase (i++); // post ++ is significant!
                 discard_buffer(bh);
             }
             else
@@ -62,26 +72,25 @@ namespace gcache
 #endif
                 return false;
             }
+
+            seqno2ptr.pop_front();
         }
 
         return true;
     }
 
     void
-    GCache::discard_tail (int64_t seqno)
+    GCache::discard_tail (seqno_t const seqno)
     {
-        seqno2ptr_t::reverse_iterator r;
-        while ((r = seqno2ptr.rbegin()) != seqno2ptr.rend() &&
-               r->first > seqno)
+        while (seqno2ptr.index_back() > seqno && !seqno2ptr.empty())
         {
-            BufferHeader* bh(ptr2BH(r->second));
+            BufferHeader* bh(ptr2BH(seqno2ptr.back()));
 
             assert(BH_is_released(bh));
-            assert(bh->seqno_g == r->first);
-            assert(bh->seqno_g > seqno);
+            assert(bh->seqno_g == seqno2ptr.index_back());
 
-            seqno2ptr.erase(--(seqno2ptr.end()));
             discard_buffer(bh);
+            seqno2ptr.pop_back();
         }
     }
 
@@ -159,7 +168,7 @@ namespace gcache
             {
                 if (gu_unlikely(!discard_seqno(bh->seqno_g)))
                 {
-                    new_released = (seqno2ptr.begin()->first - 1);
+                    new_released = (bh->seqno_g - 1);
                     assert(seqno_released <= new_released);
                 }
             }
@@ -173,13 +182,6 @@ namespace gcache
         }
         rb.assert_size_free();
 
-#ifndef NDEBUG
-        if (params.debug())
-        {
-            log_info << "GCache::free_common(): seqno_released: "
-                     << seqno_released << " -> " << new_released;
-        }
-#endif
         seqno_released = new_released;
     }
 
@@ -193,8 +195,16 @@ namespace gcache
 
 #ifndef NDEBUG
             if (params.debug()) { log_info << "GCache::free() " << bh; }
+            seqno_t const old_sr(seqno_released);
 #endif
             free_common (bh);
+#ifndef NDEBUG
+            if (params.debug())
+            {
+                log_info << "GCache::free() seqno_released: "
+                         << old_sr << " -> " << seqno_released;
+            }
+#endif
         }
         else {
             log_warn << "Attempt to free a null pointer";
