@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2012 Codership Oy <info@codership.com>
+ * Copyright (C) 2009-2018 Codership Oy <info@codership.com>
  */
 
 #include "common/common.h"
@@ -290,7 +290,21 @@ void gcomm::ViewState::write_file() const
         fclose(fout);
         return ;
     }
-    // fflush is called inside.
+
+    if (fflush(fout) != 0) {
+        log_warn << "fflush file(" << tmp << ") failed("
+                 << strerror(errno) << ")";
+        fclose(fout);
+        return ;
+    }
+
+    if (fsync(fileno(fout)) < 0) {
+        log_warn << "fsync file(" << tmp << ") failed("
+                 << strerror(errno) << ")";
+        fclose(fout);
+        return ;
+    }
+
     if (fclose(fout) != 0){
         log_warn << "close file(" << tmp << ") failed("
                  << strerror(errno) << ")";
@@ -308,8 +322,17 @@ void gcomm::ViewState::write_file() const
 bool gcomm::ViewState::read_file()
 {
     if (access(file_name_.c_str(), R_OK) != 0) {
-        log_warn << "access file(" << file_name_ << ") failed("
-                 << strerror(errno) << ")";
+        int const errn(errno);
+        std::ostringstream msg;
+        msg << "access file(" << file_name_ << ") failed("
+            << strerror(errn) << ")";
+        if (ENOENT == errn) {
+            // absence of a file should be only a notice since it is removed
+            // on graceful shutdown, so it is an expected situation
+            log_info << msg.str();
+        } else {
+            log_warn << msg.str();
+        }
         return false;
     }
     try {
