@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 Codership Oy <info@codership.com>
+ * Copyright (C) 2008-2021 Codership Oy <info@codership.com>
  *
  * $Id$
  */
@@ -685,7 +685,8 @@ _release_flow_control (gcs_conn_t* conn)
 static void
 gcs_become_primary (gcs_conn_t* conn)
 {
-    assert(conn->join_seqno == GCS_SEQNO_ILL ||
+    assert(conn->join_seqno <= 0             ||
+           conn->state == GCS_CONN_PRIMARY   ||
            conn->state == GCS_CONN_JOINER    ||
            conn->state == GCS_CONN_OPEN /* joiner that has received NON_PRIM */);
 
@@ -1525,7 +1526,7 @@ long gcs_destroy (gcs_conn_t *conn)
     gu_cond_t tmp_cond;
     gu_cond_init (&tmp_cond, NULL);
 
-    if ((err = gcs_sm_enter (conn->sm, &tmp_cond, false, true))) // need an error here
+    if (!(err = gcs_sm_enter (conn->sm, &tmp_cond, false, true))) // need an error here
     {
         if (GCS_CONN_CLOSED != conn->state)
         {
@@ -1538,8 +1539,7 @@ long gcs_destroy (gcs_conn_t *conn)
             return -EBADFD;
         }
 
-        /* this should cancel all recv calls */
-        gu_fifo_destroy (conn->recv_q);
+        gcs_sm_leave (conn->sm);
 
         gcs_shift_state (conn, GCS_CONN_DESTROYED);
 //DELETE        conn->err   = -EBADFD;
@@ -1547,12 +1547,12 @@ long gcs_destroy (gcs_conn_t *conn)
          * to acquire the lock and give up gracefully */
     }
     else {
-        gcs_sm_leave (conn->sm);
-        gu_cond_destroy (&tmp_cond);
-        err = -EBADFD;
-        return err;
+        gu_debug("gcs_destroy: gcs_sm_enter() err = %d", err);
+        // We should still cleanup resources
     }
 
+    gu_fifo_destroy (conn->recv_q);
+
     gu_cond_destroy (&tmp_cond);
     gcs_sm_destroy (conn->sm);
 
@@ -1620,6 +1620,12 @@ long gcs_caused(gcs_conn_t* conn, gcs_seqno_t& seqno)
     return gcs_core_caused(conn->core, seqno);
 }
 
+static inline bool
+fc_active(gcs_conn_t* conn)
+{
+    return conn->stop_count > 0;
+}
+
 /* Puts action in the send queue and returns after it is replicated */
 long gcs_replv (gcs_conn_t*          const conn,      //!<in
                 const struct gu_buf* const act_in,    //!<in
@@ -1663,9 +1669,8 @@ long gcs_replv (gcs_conn_t*          const conn,      //!<in
             // if (conn->state >= GCS_CONN_CLOSE) or (act_ptr == NULL)
             // ret will be -ENOTCONN
             if ((ret = -EAGAIN,
-                 conn->upper_limit >= conn->queue_len ||
-                 act->type         != GCS_ACT_TORDERED)         &&
-                (ret = -ENOTCONN, GCS_CONN_OPEN >= conn->state) &&
+                 !fc_active(conn) || act->type != GCS_ACT_TORDERED) &&
+                (ret = -ENOTCONN, GCS_CONN_OPEN >= conn->state)     &&
                 (act_ptr = (struct gcs_repl_act**)gcs_fifo_lite_get_tail (conn->repl_q)))
             {
                 *act_ptr = &repl_act;
@@ -2018,7 +2023,18 @@ gcs_join (gcs_conn_t* conn, gcs_seqno_t const seqno)
 {
     assert(conn->join_seqno <= seqno || seqno < 0);
 
-    if (seqno < 0 || seqno >= conn->join_seqno)
+    /*
+     * Always allow sending of join messages when not in JOINER state.
+     * This is required for correct handling of desync counter,
+     * especially in DONOR state:
+     * If the DONOR does desync in combination with SST donation, the
+     * gcs_join() calls from resync() and sst_sent() might
+     * come with out of order seqnos, leaving the desync_count in gcs_group
+     * permanently in non-zero value. In this case the node will not become
+     * synced again unless it is temporarily removed from the group.
+     */
+    if (conn->state != GCS_CONN_JOINER ||
+        seqno < 0 || seqno >= conn->join_seqno)
     {
         conn->join_seqno   = seqno;
         conn->need_to_join = true;
@@ -2055,6 +2071,8 @@ gcs_get_stats (gcs_conn_t* conn, struct gcs_stats* stats)
     stats->fc_ssent    = conn->stats_fc_stop_sent;
     stats->fc_csent    = conn->stats_fc_cont_sent;
     stats->fc_received = conn->stats_fc_received;
+    stats->fc_active   = fc_active(conn);
+    stats->fc_requested= conn->stop_sent_ > 0;
 }
 
 void
