@@ -420,14 +420,6 @@ typedef struct gcs_act_conf {
                                 *  incoming address, 8-byte cached seqno) */
 } gcs_act_conf_t;
 
-typedef struct gcs_backend_stats {
-    struct stats_t {
-        const char* key;
-        const char* value;
-    }* stats;
-    void* ctx;
-} gcs_backend_stats_t;
-
 struct gcs_stats
 {
     double    send_q_len_avg; //! average send queue length per send call
@@ -444,7 +436,8 @@ struct gcs_stats
     int       send_q_len;     //! current send queue length
     int       send_q_len_max; //! maximum send queue length
     int       send_q_len_min; //! minimum send queue length
-    gcs_backend_stats_t backend_stats; //! backend stats.
+    bool      fc_active;      //! flow control is currently active
+    bool      fc_requested;   //! flow control is requested by this node
 };
 
 /*! Fills stats struct */
