@@ -35,11 +35,13 @@ Options:
     --dl            set debug level for Scons build (1, implies -c)
     -r|--release    release number
     -m32/-m64       build 32/64-bit binaries for x86
-    -p|--package    build RPM and DEB packages at the end.
+    -p|--package    build RPM packages at the end (DEB not supported).
     --with-spread   configure build with spread backend (implies -c to gcs)
     --source        build source packages
     --sb            skip actual build, use the existing binaries
-    --scons         build using Scons build system (yes)
+    --cmake         build using CMake build system (yes)
+    --co            CMake options
+    --scons         build using Scons build system (no)
     --so            Sconscript option
     -j|--jobs       how many parallel jobs to use for Scons (1)
     "\nSet DISABLE_GCOMM/DISABLE_VSBES to 'yes' to disable respective modules"
@@ -56,7 +58,9 @@ RELEASE=${RELEASE:-""}
 SOURCE=${SOURCE:-"no"}
 DEBUG=${DEBUG:-"no"}
 DEBUG_LEVEL=${DEBUG_LEVEL:-"0"}
-SCONS=${SCONS:-"yes"}
+CMAKE=${CMAKE:-"yes"}
+CMAKE_OPTS=${CMAKE_OPTS:-""}
+SCONS=${SCONS:-"no"}
 SCONS_OPTS=${SCONS_OPTS:-""}
 export JOBS=${JOBS:-"$(get_cores)"}
 SCRATCH=${SCRATCH:-"no"}
@@ -88,12 +92,9 @@ elif [ "$OS" == "FreeBSD" ]; then
   EXTRA_SYSROOT=/usr/local
 fi
 
-which dpkg >/dev/null 2>&1 && DEBIAN=${DEBIAN:-1} || DEBIAN=${DEBIAN:-0}
-
 if [ "$OS" == "FreeBSD" ]; then
-    CC=${CC:-"gcc48"}
-    CXX=${CXX:-"g++48"}
-    LD_LIBRARY_PATH=${LD_LIBRARY_PATH:-"/usr/local/lib/$(basename $CC)"}
+    CC=${CC:-"clang"}
+    CXX=${CXX:-"clang++"}
 else
     CC=${CC:-"gcc"}
     CXX=${CXX:-"g++"}
@@ -115,9 +116,9 @@ last_stage="galera"
 gainroot=""
 TARGET=${TARGET:-""} # default target
 
-while test $# -gt 0 
+while test $# -gt 0
 do
-    case $1 in 
+    case $1 in
         --stage)
             initial_stage=$2
             shift
@@ -178,8 +179,17 @@ do
         --sb)
             SKIP_BUILD="yes"
             ;;
+        --cmake)
+            CMAKE="yes"
+            SCONS="no"
+            ;;
+        --co)
+            CMAKE_OPTS="$CMAKE_OPTS $2"
+            shift
+            ;;
         --scons)
             SCONS="yes"
+            CMAKE="no"
             ;;
         --so)
             SCONS_OPTS="$SCONS_OPTS $2"
@@ -207,6 +217,11 @@ done
 # check whether sudo accepts -E to preserve environment
 if [ "$PACKAGE" == "yes" ]
 then
+    if which dpkg >/dev/null 2>&1
+    then
+        echo "Error: Package build not supported on Debian, use dpkg-buildpackage"
+        exit 1
+    fi
     echo "testing sudo"
     if sudo -E $true >/dev/null 2>&1
     then
@@ -290,9 +305,7 @@ build_packages()
     $SUDO rm -rf $ARCH
 
     set +e
-    if [ $DEBIAN -ne 0 ]; then # build DEB
-        ./deb.sh $GALERA_VER
-    elif [ "$OS" == "FreeBSD" ]; then
+    if [ "$OS" == "FreeBSD" ]; then
         if test "$NO_STRIP" != "yes"; then
             strip $build_base/{garb/garbd,libgalera_smm.so}
         fi
@@ -305,9 +318,7 @@ build_packages()
     set -e
 
     popd
-    if [ $DEBIAN -ne 0 ]; then
-        mv -f $PKG_DIR/$ARCH/*.deb ./
-    elif [ "$OS" == "FreeBSD" ]; then
+    if [ "$OS" == "FreeBSD" ]; then
         mv -f $PKG_DIR/*.tbz ./
     else
         mv -f $PKG_DIR/*.rpm ./
@@ -364,14 +375,41 @@ pushd "$build_base"
     # trim spaces (sed is not working on Solaris, so using bash built-in)
     GALERA_REV=${GALERA_REV//[[:space:]]/}
 #fi
-popd
 
-#if [ -z "$RELEASE" ]
-#then
-#    RELEASE=$GALERA_REV
-#fi
+if [ -z "$RELEASE" ]
+then
+    source GALERA_VERSION
+    RELEASE="$GALERA_VERSION_WSREP_API.$GALERA_VERSION_MAJOR.$GALERA_VERSION_MINOR"
+fi
 
-if [ "$SCONS" == "yes" ] # Build using Scons
+if [ "$CMAKE" == "yes" ] # Build using CMake
+then
+    cmake_args="$CMAKE_OPTS -DGALERA_REVISION=$GALERA_REV"
+    [ -n "$TARGET"        ] && \
+        echo "WARN: TARGET=$TARGET ignored by CMake build"
+    [ -n "$RELEASE"       ] && \
+        echo "WARN: RELEASE=$RELEASE ignored by CMake build"
+    [ "$DEBUG" == "yes"   ] && cmake_args="$cmake_args -DCMAKE_BUILD_TYPE=Debug"
+    [ -n "$EXTRA_SYSROOT" ] && \
+        echo "EXTRA_SYSROOT=$EXTRA_SYSROOT ignored by CMake build"
+
+    if [ "$SCRATCH" == "yes" ]
+    then
+        (cd $build_base && make clean) || :
+        rm -f $build_base/CMakeCache.txt
+        cmake $cmake_args $build_base
+    fi
+
+    if [ "$SKIP_BUILD" != "yes" ]
+    then
+        make -j $JOBS VERBOSE=1
+    fi
+
+    if [ $RUN_TESTS ]
+    then
+        make test ARGS=-V
+    fi
+elif [ "$SCONS" == "yes" ] # Build using Scons
 then
     # Scons variant dir, defaults to GALERA_SRC
     export SCONS_VD=$build_base
@@ -407,4 +445,4 @@ then
     build_sources
 fi
 
-
+popd # $build_base
